#
# Copyright (C) 2019 Authlete, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific
# language governing permissions and limitations under the
# License.


from .authorization_request_handler_spi_adapter import AuthorizationRequestHandlerSpiAdapter
from .no_interaction_handler_spi                import NoInteractionHandlerSpi


class NoInteractionHandlerSpiAdapter(
    AuthorizationRequestHandlerSpiAdapter, NoInteractionHandlerSpi):
    """An empty implementation of NoInteractionHandlerSpi.
    """


    def isUserAuthenticated(self):
        return False
